/* *****************************************************************************
 * libg1m/format/std/lang.h -- the G1M language file format description.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libg1m.
 * libg1m is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libg1m is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libg1m; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#ifndef LIBG1M_FORMAT_STD_LANG_H
# define LIBG1M_FORMAT_STD_LANG_H
# include <stdint.h>
# pragma pack(1)

/* ************************************************************************** */
/*  G1L - Language files for fx calculators                                   */
/* ************************************************************************** */
/* It all starts with a header: */

typedef struct g1m_lang_subheader_s {
	/* identifier: is "PowerUSB" on original things */
	uint8_t  g1m_lang_subheader_identifier[8];

	/* OS information (raw binary format), e.g. 0x02,0x09 for 2.09. */
	uint8_t  g1m_lang_subheader_os_major;
	uint8_t  g1m_lang_subheader_os_minor;

	/* unknown bytes */
	uint8_t  g1m_lang_subheader__unknown[2];

	/* message count */
	uint16_t g1m_lang_subheader_count;
} g1m_lang_subheader_t;

/* Then comes a list of offsets, and the messages.
 *
 * Each element of the offsets list is 16-bytes long.
 * It is relative to the first element (which starts right after the
 * offsets list - no alignment).
 *
 * The messages are null-terminated - once you get the offsets, you get
 * them completely. */
/* ************************************************************************** */
/*  G3L - Language files for Prizm                                            */
/* ************************************************************************** */
/* G3L and G3N start with the StandardHeader and the Standard Subheader,
 * then the Prizm-specific subheader. After this, both the G3L and G3N
 * have this language header: */

typedef struct g1m_prizm_lang_subheader_s {
	/* sequence: '4C 59 37 35 35 00 00 00' (LY755   ) */
	uint8_t  g1m_prizm_lang_subheader_sequence[8];

	/* unknown: 0x02 */
	uint8_t  g1m_prizm_lang_subheader__unknown;

	/* unused byte. */
	uint8_t  g1m_prizm_lang_subheader__unused0;

	/* number of messages ("possibly 0 base indexed") */
	uint32_t g1m_prizm_lang_subheader_count;

	/* unused bytes */
	uint8_t  g1m_prizm_lang_subheader__unused1[2];
} g1m_prizm_lang_subheader_t;

/* Then we have offsets of all messages (4 bytes each),
 * then messages themselves, zero-terminated.
 *
 * And don't forget the footer (see `libg1m/format/std.h`) */

# pragma pack()
#endif /* LIBG1M_FORMAT_STD_LANG_H */
