/* *****************************************************************************
 * decode/cas/program.c -- decode a CAS program.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libg1m.
 * libg1m is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libg1m is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libg1m; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libg1m/internals.h>

/**
 *	g1m_decode_caspart_program:
 *	Decode a CAS program part.
 *
 *	@arg	handle		the handle to contribute to.
 *	@arg	buffer		the buffer to read from.
 *	@return				the error code (0 if ok).
 */

int g1m_decode_caspart_program(g1m_mcsfile_t *handle, g1m_buffer_t *buffer)
{
	/* get content */
	READ(handle->g1m_mcsfile_content, handle->g1m_mcsfile_head.g1m_mcshead_size)
	log_info("Program content is:");
	logm_info(handle->g1m_mcsfile_content,
		handle->g1m_mcsfile_head.g1m_mcshead_size);

	/* check the sum */
	uint8_t checksum; READ(&checksum, 1)
	uint8_t csum = ~g1m_checksum8(handle->g1m_mcsfile_content,
		handle->g1m_mcsfile_head.g1m_mcshead_size) + 1;
	if (csum != checksum) {
		log_error("Checksum mismatch: expected 0x%02X, got 0x%02X",
			csum, checksum);
		return (g1m_error_checksum);
	}

	/* everything went well :) */
	handle->g1m_mcsfile_head.g1m_mcshead_flags &= ~g1m_mcsflag_unfinished;
	return (0);
}
