/* *****************************************************************************
 * encode/addin.c -- encode an addin.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libg1m.
 * libg1m is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libg1m is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libg1m; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libg1m/internals.h>

/**
 *	g1m_announce_addin:
 *	Announce an add-in's size.
 *
 *	@arg	handle		the handle.
 *	@arg	size		the size to feed.
 *	@return				the error code (0 if ok).
 */

int g1m_announce_addin(g1m_handle_t *handle, size_t *size)
{
	*size += sizeof(g1m_standard_header_t) + sizeof(g1m_addin_subheader_t);
	*size += handle->g1m_handle_size;
	return (0);
}

/**
 *	g1m_encode_addin:
 *	Encode an add-in.
 *
 *	@arg	handle		the handle.
 *	@arg	buffer		the buffer to which to write to.
 *	@return				the error code (0 if ok).
 */

int g1m_encode_addin(g1m_handle_t *handle, g1m_buffer_t *buffer)
{
	size_t i; uint8_t *b;

	/* make the StandardHeader up */
	size_t filesize = 0; g1m_announce_addin(handle, &filesize);
	g1m_standard_header_t std = {
		.g1m_standard_header_main_id = "USBPower",
		.g1m_standard_header_subtype = "\xF3\x00\x10\x00\x10\x00",
		.g1m_standard_header_filesize = htobe32((uint32_t)filesize),
		.g1m_standard_header_control = (filesize + 0x41) & 0xFF,
		.g1m_standard_header_control2 = (filesize + 0xB8) & 0xFF,
		.g1m_standard_header_number = 0xFFFF};

	/* reverse it and output it */
	b = (void*)&std;
	for (i = 0; i < sizeof(g1m_standard_header_t); i++) b[i] = ~b[i];
	DWRITE(std)

	/* make the add-in subheader up and output it */
	filesize -= sizeof(g1m_standard_header_t);
	g1m_addin_subheader_t sub = {
		.g1m_addin_subheader_estrips_count = 0,
		.g1m_addin_subheader_filesize = htobe32(filesize)};
	strncpy((char*)sub.g1m_addin_subheader_title,
		handle->g1m_handle_title, 8);
	strncpy((char*)sub.g1m_addin_subheader_internal_name,
		handle->g1m_handle_intname, 8);
	g1m_encode_version(&handle->g1m_handle_version,
		(char*)sub.g1m_addin_subheader_version);
	g1m_encode_date(&handle->g1m_handle_creation_date,
		(char*)sub.g1m_addin_subheader_creation_date);
	g1m_encode_picture((const uint32_t**)handle->g1m_handle_icon_unsel,
		g1m_pictureformat_1bit, sub.g1m_addin_subheader_icon,
		G1A_ICON_WIDTH, G1A_ICON_HEIGHT);
	DWRITE(sub)

	/* output the content */
	WRITE(handle->g1m_handle_content, handle->g1m_handle_size)

	/* no error! */
	return (0);
}
