/* *****************************************************************************
 * encode/mcs/var.c -- encode an MCS variable/alpha memory file.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libg1m.
 * libg1m is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libg1m is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libg1m; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libg1m/internals.h>

/**
 *	g1m_announce_mcs_var:
 *	Announce alpha memory size.
 *
 *	@arg	handle		the handle to find the size of.
 *	@arg	sz			the size to contribute to.
 *	@return				the error code (0 if ok).
 */

int g1m_announce_mcs_var(const g1m_mcsfile_t *handle, size_t *sz)
{
	*sz += sizeof(g1m_mcsbcd_t) * handle->g1m_mcsfile_head.g1m_mcshead_count;
	return (0);
}

/**
 *	g1m_encode_mcs_var:
 *	Encode alpha memory.
 *
 *	@arg	handle		the handle to encode.
 *	@arg	buffer		the buffer to write to.
 *	@return				the error code (0 if ok).
 */

int g1m_encode_mcs_var(const g1m_mcsfile_t *handle, g1m_buffer_t *buffer)
{
	int i;

	/* encode each variable */
	for (i = 0; i < handle->g1m_mcsfile_head.g1m_mcshead_count; i++) {
		/* write real and imaginary part */
		g1m_mcsbcd_t rawbcd;
		g1m_bcd_tomcs(&handle->g1m_mcsfile_vars[i].g1m_mcscell_real, &rawbcd);
		DWRITE(rawbcd)
		g1m_bcd_tomcs(&handle->g1m_mcsfile_vars[i].g1m_mcscell_imgn, &rawbcd);
		DWRITE(rawbcd)
	}

	/* no error! */
	return (0);
}
